open Context
open Structure

let _PAT_WS = Str.regexp "[\032\009\013\010]+"

(* Transform a tree by removing extraneous spaces within a single TEXT *)
let transform tree_in =
	let rec transform_r tr context value =
		match tr with
		| [] ->
			List.rev value
		| hd :: rest ->
			match hd with
			| Text st ->
				if context.tt then
					transform_r rest context (hd :: value)
				else begin
					let newtext = Str.global_replace _PAT_WS " " st in
					transform_r rest context ((Text newtext) :: value)
				end
			| Bold seq ->
				let newhd = transform_r seq {context with b=true} [] in
				transform_r rest context ((Bold newhd) :: value)
			| Emphasis seq ->
				let ctx =
					if context.s then context
					else {context with em=not context.em}
				in
				let newhd = transform_r seq ctx [] in
				transform_r rest context ((Emphasis newhd) :: value)
			| Italic seq ->
				let newhd = transform_r seq {context with i=true} [] in
				transform_r rest context ((Italic newhd) :: value)
			| Plain seq ->
				let newhd = transform_r seq
						{context with s=false; em=false;
						 i=false; b=false;
						 tt=false; u=0} [] in
				transform_r rest context ((Plain newhd) :: value)
			| Strong seq ->
				let newhd = transform_r seq
						{context with s=true; em=false} [] in
				transform_r rest context ((Strong newhd) :: value)
			| Typewriter seq ->
				let newhd = transform_r seq {context with tt=true} [] in
				transform_r rest context ((Typewriter newhd) :: value)
			| Underline seq ->
				let newval =
					if context.u < 3 then (context.u + 1)
					else context.u
				in
				let newhd = transform_r seq {context with u=newval} [] in
				transform_r rest context ((Underline newhd) :: value)
			| Size (sz,seq) ->
				let newhd = transform_r seq {context with size=sz} [] in
				transform_r rest context ((Size (sz,newhd)) :: value)
			| Red seq ->
				let newhd = transform_r seq {context with color=R} [] in
				transform_r rest context ((Red newhd) :: value)
			| Green seq ->
				let newhd = transform_r seq {context with color=G} [] in
				transform_r rest context ((Green newhd) :: value)
			| Blue seq ->
				let newhd = transform_r seq {context with color=B} [] in
				transform_r rest context ((Blue newhd) :: value)
			| Cyan seq ->
				let newhd = transform_r seq {context with color=C} [] in
				transform_r rest context ((Cyan newhd) :: value)
			| Magenta seq ->
				let newhd = transform_r seq {context with color=M} [] in
				transform_r rest context ((Magenta newhd) :: value)
			| Yellow seq ->
				let newhd = transform_r seq {context with color=Y} [] in
				transform_r rest context ((Yellow newhd) :: value)
			| Black seq ->
				let newhd = transform_r seq {context with color=K} [] in
				transform_r rest context ((Black newhd) :: value)
			| White seq ->
				let newhd = transform_r seq {context with color=W} [] in
				transform_r rest context ((White newhd) :: value)
	in
	let result = transform_r tree_in
			{b=false; em=false; i=false; s=false; tt=false;
			u=0; size=0; color=Any} [] in
	result
