open Context
open Structure

(* Transform a tree by removing empty segments *)
let transform tree_in =
	let rec transform_r tr context value =
		match tr with
		| [] ->
			List.rev value
		| hd :: rest ->
			match hd with
			| Text st ->
				transform_r rest context ((Text st) :: value)
			| Bold seq ->
				if seq = [] then transform_r rest context value
				else begin
					let newhd = transform_r seq {context with b=true} [] in
					transform_r rest context ((Bold newhd) :: value)
				end
			| Emphasis seq ->
				if seq = [] then transform_r rest context value
				else begin
					let ctx =
						if context.s then context
						else {context with em=not context.em}
					in
					let newhd = transform_r seq ctx [] in
					transform_r rest context ((Emphasis newhd) :: value)
				end
			| Italic seq ->
				if seq = [] then transform_r rest context value
				else begin
					let newhd = transform_r seq {context with i=true} [] in
					transform_r rest context ((Italic newhd) :: value)
				end
			| Plain seq ->
				if seq = [] then transform_r rest context value
				else begin
					let newhd = transform_r seq
							{context with s=false; em=false;
							 i=false; b=false;
							 tt=false; u=0} [] in
					transform_r rest context ((Plain newhd) :: value)
				end
			| Strong seq ->
				if seq = [] then transform_r rest context value
				else begin
					let newhd = transform_r seq
							{context with s=true; em=false} [] in
					transform_r rest context ((Strong newhd) :: value)
				end
			| Typewriter seq ->
				if seq = [] then transform_r rest context value
				else begin
					let newhd = transform_r seq {context with tt=true} [] in
					transform_r rest context ((Typewriter newhd) :: value)
				end
			| Underline seq ->
				if seq = [] then transform_r rest context value
				else begin
					let newval =
						if context.u < 3 then (context.u + 1)
						else context.u
					in
					let newhd = transform_r seq {context with u=newval} [] in
					transform_r rest context ((Underline newhd) :: value)
				end
			| Size (sz,seq) ->
				if seq = [] then transform_r rest context value
				else begin
					let newhd = transform_r seq {context with size=sz} [] in
					transform_r rest context ((Size (sz,newhd)) :: value)
				end
			| Red seq ->
				if seq = [] then transform_r rest context value
				else begin
					let newhd = transform_r seq {context with color=R} [] in
					transform_r rest context ((Red newhd) :: value)
				end
			| Green seq ->
				if seq = [] then transform_r rest context value
				else begin
					let newhd = transform_r seq {context with color=G} [] in
					transform_r rest context ((Green newhd) :: value)
				end
			| Blue seq ->
				if seq = [] then transform_r rest context value
				else begin
					let newhd = transform_r seq {context with color=B} [] in
					transform_r rest context ((Blue newhd) :: value)
				end
			| Cyan seq ->
				if seq = [] then transform_r rest context value
				else begin
					let newhd = transform_r seq {context with color=C} [] in
					transform_r rest context ((Cyan newhd) :: value)
				end
			| Magenta seq ->
				if seq = [] then transform_r rest context value
				else begin
					let newhd = transform_r seq {context with color=M} [] in
					transform_r rest context ((Magenta newhd) :: value)
				end
			| Yellow seq ->
				if seq = [] then transform_r rest context value
				else begin
					let newhd = transform_r seq {context with color=Y} [] in
					transform_r rest context ((Yellow newhd) :: value)
				end
			| Black seq ->
				if seq = [] then transform_r rest context value
				else begin
					let newhd = transform_r seq {context with color=K} [] in
					transform_r rest context ((Black newhd) :: value)
				end
			| White seq ->
				if seq = [] then transform_r rest context value
				else begin
					let newhd = transform_r seq {context with color=W} [] in
					transform_r rest context ((White newhd) :: value)
				end
	in
	let result = transform_r tree_in
			{b=false; em=false; i=false; s=false; tt=false;
			u=0; size=0; color=Any} [] in
	result
