let m = Mutex.create()

type t = {
	mutable tree:Structure.t list;
	mutable value:string;
}
let result = { tree=[Structure.Text ""]; value=""; }

let optimizations = [
	Useless_em.transform;
	Empty_seg.transform;
	Compress_text.transform;
	Compress_white.transform;
	Redundant.transform;
	Remove_white.transform;
]

let main parent =
	let lexbuf = Lexing.from_channel stdin in
	(* Stuff a copy of the original in the "best" buffer so we will
	 * always have *SOME* result
	 *)
	let treerep = Parser.main Lexer.token lexbuf in
	let strrep = Structure.print_t treerep in
	Mutex.lock m;
	result.tree <- treerep;
	result.value <- strrep;
	Mutex.unlock m;

	(* Now that we have a fallback position we can start trying crap *)
	let rec try_list curtree =
		let rec try_one optlist ct =
			match optlist with
			| [] -> ct
			| opt :: rest ->
				let newt = opt ct in
				let news = Structure.print_t newt in
				let newl = String.length news in
				Mutex.lock m;
				if newl < (String.length result.value) then
					begin
					result.tree <- newt;
					result.value <- news;
					end;
				Mutex.unlock m;
				try_one rest newt
		in
		let curstr = Structure.print_t curtree in
		let newtree = try_one optimizations curtree in
		let newstr = Structure.print_t newtree in
		if newstr = curstr then ()
		else try_list newtree;
	in
	try_list treerep;
	Unix.kill (Unix.getpid()) Sys.sigusr1;
	Thread.join parent


let _ =
	try
		let timelimit = int_of_string Sys.argv.(1) in
		let worker = Thread.create main (Thread.self()) in
		ignore(Unix.alarm (timelimit-5));
		ignore(Thread.wait_signal [ Sys.sigalrm; Sys.sigusr1 ]);
		Mutex.lock m;
		print_string result.value;
		Mutex.unlock m;
		exit 0
	with Invalid_argument "Array.get" ->
		print_string "Usage: runme timelimit";
		print_newline();
		exit 1
