type t =
	| Text of string
	| Bold of t list
	| Emphasis of t list
	| Italic of t list
	| Plain of t list
	| Strong of t list
	| Typewriter of t list
	| Underline of t list
	| Size of int * t list
	| Red of t list
	| Green of t list
	| Blue of t list
	| Cyan of t list
	| Magenta of t list
	| Yellow of t list
	| Black of t list
	| White of t list


(* Pretty-print a [t list] onto stdout *)
let rec print_t thinglist =
		let rec print_one thinglist slist =
			match thinglist with
			| [] -> List.rev slist
			| Text s :: rest -> print_one rest (s :: slist)
			| Bold b :: rest ->
				print_one rest (("<B>" ^ (print_t b) ^ "</B>") :: slist)
			| Emphasis b :: rest ->
				print_one rest (("<EM>" ^ (print_t b) ^ "</EM>") :: slist)
			| Italic b :: rest ->
				print_one rest (("<I>" ^ (print_t b) ^ "</I>") :: slist)
			| Plain b :: rest ->
				print_one rest (("<PL>" ^ (print_t b) ^ "</PL>") :: slist)
			| Strong b :: rest ->
				print_one rest (("<S>" ^ (print_t b) ^ "</S>") :: slist)
			| Typewriter b :: rest ->
				print_one rest (("<TT>" ^ (print_t b) ^ "</TT>") :: slist)
			| Underline b :: rest ->
				print_one rest (("<U>" ^ (print_t b) ^ "</U>") :: slist)
			| Size (s,b) :: rest ->
				let bstr = Printf.sprintf "<%d>" s in
				let estr = Printf.sprintf "</%d>" s in
				print_one rest ((bstr ^ (print_t b) ^ estr) :: slist)
			| Red b :: rest ->
				print_one rest (("<r>" ^ (print_t b) ^ "</r>") :: slist)
			| Green b :: rest ->
				print_one rest (("<g>" ^ (print_t b) ^ "</g>") :: slist)
			| Blue b :: rest ->
				print_one rest (("<b>" ^ (print_t b) ^ "</b>") :: slist)
			| Cyan b :: rest ->
				print_one rest (("<c>" ^ (print_t b) ^ "</c>") :: slist)
			| Magenta b :: rest ->
				print_one rest (("<m>" ^ (print_t b) ^ "</m>") :: slist)
			| Yellow b :: rest ->
				print_one rest (("<y>" ^ (print_t b) ^ "</y>") :: slist)
			| Black b :: rest ->
				print_one rest (("<k>" ^ (print_t b) ^ "</k>") :: slist)
			| White b :: rest ->
				print_one rest (("<w>" ^ (print_t b) ^ "</w>") :: slist)
		in
		List.fold_left (^) "" (print_one thinglist [])
